#include <QApplication>
#include <QWidget>
#include <QPainter>
#include <QVBoxLayout>
#include <QLabel>
#include <QDebug>
#include <QTextLayout>
#include <QTextLine>

// To see if floating point vs int makes a difference (it doesn't)
//#define QPointF QPoint
//#define QSizeF QSize
//#define QRectF QRect
//#define QFontMetricsF QFontMetrics

class Window : public QWidget
{
public:
    Window() : QWidget(nullptr) {
        QFont font(this->font());
        //font.setPointSize(12);
        font.setItalic(true);
        setFont(font);
    }
protected:
    void paintEvent(QPaintEvent *) {
        QPainter painter(this);
        painter.scale(2, 2);
        QFontMetricsF fm{font()};
        const QString text = "This text is in italics";

        const int x = 10;
        double y = 10;

        auto drawOne = [&](const QRectF &textRect, const QString &hint) {
            qDebug() << textRect << hint;
            const QRectF rect(x, y, textRect.width(), textRect.height());
            QRectF outBr;
            painter.drawText(rect, Qt::TextDontClip, text, &outBr);
            //painter.drawRect(outBr.adjusted(0, 0, -1, -1));
            painter.drawRect(rect.adjusted(0, 0, -1, -1));
            //qDebug() << " outBr=" << outBr;
            painter.drawText(rect.right() + 20, rect.y() + fm.ascent(), hint);
            y += 50;
        };

        const QPointF zero(0, 0);

        //const QSizeF sizeFmWidth(fm.width(text), fm.height());
        const QSizeF sizeFmWidth(fm.horizontalAdvance(text), fm.height());
        drawOne(QRectF(zero, sizeFmWidth), "QFontMetrics::width (deprecated)");

        const QSizeF sizeSingleLine = fm.size(Qt::TextSingleLine, text);
        drawOne(QRectF(zero, sizeSingleLine), "QFontMetrics::size");

        const QSizeF sizeAdvance(fm.horizontalAdvance(text), fm.height());
        drawOne(QRectF(zero, sizeAdvance), "QFontMetrics::horizontalAdvance");

        QTextLayout layout(text, font());
        layout.beginLayout();
        QTextLine line = layout.createLine();
        layout.endLayout();
        line.setPosition( QPoint( 0, 0 ) );
        QSizeF sizeTextLayout(line.naturalTextWidth(), line.height());
        drawOne(QRectF(zero, sizeTextLayout), "QTextLayout::naturalTextWidth");

        // boundingRect is smaller, and weirder
        drawOne(fm.boundingRect(text), "QFontMetrics::boundingRect");
        // tightBoundingRect is even smaller
        drawOne(fm.tightBoundingRect(text), "QFontMetrics::tightBoundingRect");
    }
};


int main( int argc, char** argv )
{
    QApplication app(argc,argv);

    Window* window = new Window;
    window->resize(1000, 700);
    window->show();

    return app.exec();
}
